@@ -52,6 +52,14 @@ typedef struct _PangoFontDescription PangoFontDescription;
  * to a script. The fields of this structure are private to implementations
  * of a font backend. See the documentation of the corresponding getters
  * for documentation of their meaning.
+ *
+ * For an overview of the most important metrics, see:
+ *
+ * <picture>
+ *   <source srcset="fontmetrics-dark.png" media="(prefers-color-scheme: dark)">
+ *   <img alt="Font metrics" src="fontmetrics-light.png">
+ * </picture>
+
  */
 typedef struct _PangoFontMetrics PangoFontMetrics;
 
@@ -656,6 +664,10 @@ PangoLanguage **      pango_font_get_languages     (PangoFont        *font);
 #endif
 #endif
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(PangoFontFamily, g_object_unref)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(PangoFontFace, g_object_unref)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(PangoFont, g_object_unref)
+
 G_END_DECLS
 
 #endif /* __PANGO_FONT_H__ */
